#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `potranslator` package."""

import pytest
import sys
from os.path import isfile, join, getmtime
from os import remove

from click.testing import CliRunner

from potranslator import PoTranslator
from potranslator import cli


is_python2 = sys.version_info < (3, 0)

class TestPoTranslator:
    translator = PoTranslator(pot_dir='./tests/test_data/test_pot_files', locale_dir='./tests/test_data/locale')
    test_po_file = './tests/test_data/empty_test.po'
    test_pot_file = './tests/test_data/test_pot_files/test-usage.pot'
    test_languages = ('es', 'fr', 'it', 'pt', 'ro')

    def test_failed_translate_unsupported_language(self):
        """

        :return:
        """
        with pytest.raises(ValueError):
            failed_translation = self.translator.translate(self.test_po_file, 'sp')
        return

    def test_failed_translate_auto_detect(self):
        """

        :return:
        """
        with pytest.raises(ValueError):
            failed_translation = self.translator.translate(self.test_po_file)
        return

    def test_translate(self):
        """

        :return:
        """
        translation = self.translator.translate(self.test_po_file, 'es')
        assert all([entry.msgstr != '' for entry in translation])
        if not is_python2:
            assert translation[0].msgstr == 'Créditos'
        else:
            assert translation[0].msgstr.encode('utf-8') == 'Créditos'
        return

    def test_translate_auto_save(self):
        """

        :return:
        """
        modif_time = getmtime(self.test_po_file)
        translation = self.translator.translate(self.test_po_file, 'es', auto_save=True)
        last_modif_time = getmtime(self.test_po_file)
        assert modif_time < last_modif_time
        return

    def test_translate_all_locale(self):
        """

        :return:
        """
        translations = self.translator.translate_all_locale()
        assert not 'sp' in translations
        assert all(k in translations for k in self.test_languages)
        assert all('authors.po' in k for k in list(translations.values()))
        if not is_python2:
            assert translations['es']['authors.po'][0].msgstr == 'Créditos'
        else:
            assert translations['es']['authors.po'][0].msgstr.encode('utf-8') == 'Créditos'
        return

    def test_translate_from_pot(self):
        """

        :return:
        """
        translations = self.translator.translate_from_pot(self.test_pot_file, target_langs=self.test_languages)
        assert all(k in translations for k in self.test_languages)
        if not is_python2:
            assert translations['es'][0].msgstr == 'Uso'
        else:
            assert translations['es'][0].msgstr.encode('utf-8') == 'Uso'
        assert isfile(join(self.translator.locale_dir, '/'.join(('es', 'LC_MESSAGES', 'test-usage.po'))))
        for locale in self.test_languages:
            remove(join(self.translator.locale_dir, '/'.join((locale, 'LC_MESSAGES', 'test-usage.po'))))
        return

    def test_translate_all_pot(self):
        """

        :return:
        """
        translations = self.translator.translate_all_pot(target_langs=self.test_languages)
        assert all(k in translations['test-authors.pot'] for k in self.test_languages)
        assert all(k in translations['test-usage.pot'] for k in self.test_languages)

        if not is_python2:
            assert translations['test-authors.pot']['es'][0].msgstr == 'Créditos'
        else:
            assert translations['test-authors.pot']['es'][0].msgstr.encode('utf-8') == 'Créditos'
        for locale in self.test_languages:
            for file in ('test-authors.po', 'test-usage.po'):
                remove(join(self.translator.locale_dir, '/'.join((locale, 'LC_MESSAGES', file))))
        return



def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'potranslator.cli.main' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output



# -*- coding: utf-8 -*-

"""Main module."""

from os import listdir, makedirs
from os.path import isfile, join, exists
from . import polib, Translator, pkg_resources, json
from . import SUPPORTED_LANGUAGES, __version__
from collections import defaultdict
from copy import deepcopy
from codecs import open
import sys

_RESOURCE_PACKAGE = __name__

is_python2 = sys.version_info < (3, 0)




class PoTranslator:
    """

    :param pot_dir:
    :param locale_dir:
    """
    def __init__(self, pot_dir=None, locale_dir=None):
        self.pot_dir = pot_dir
        self.locale_dir = locale_dir
        self.translator = Translator()
        return

    def translate(self, file_name, target_lang='auto', src_lang='auto', encoding='utf-8', auto_save=False):
        """


        :param file_name:
        :param target_lang:
        :param src_lang:
        :param encoding:
        :param auto_save:
        :return: POFile.
        """
        po = polib.pofile(file_name, encoding=encoding)
        if target_lang == 'auto':
            try:
                target_lang = po.metadata['Language']
            except KeyError:
                raise ValueError(_('potranslator could not auto-detect the desired translation language for the file {0}.\nPlease provide a target language.').format(file_name))
        if target_lang not in SUPPORTED_LANGUAGES:
            raise ValueError(_('Unsupported language. To see the list of supported languages type potranslator -h'))
        untranslated = [elmt for elmt in po if elmt.msgstr == '' and not elmt.obsolete]
        translations = self.translator.translate([elmt.msgid for elmt in untranslated], src=src_lang, dest=target_lang)
        print(_('{0} translations for the file {1} have been succesfully retrieved').format(SUPPORTED_LANGUAGES[target_lang], file_name))
        for entry, translation in zip(untranslated, translations):
            entry.msgstr = translation.text
        po.metadata['Translated-By'] = 'potranslator {0}'.format(__version__)
        if auto_save:
            po.save(file_name)
            print(_('The file {1} has been succesfully translated in {0} and saved.').format(SUPPORTED_LANGUAGES[target_lang], file_name))
        else:
            print(_('The file {1} has been succesfully translated in {0}.').format(SUPPORTED_LANGUAGES[target_lang], file_name))
        return po

    def translate_all_locale(self, src_lang='auto', encoding='utf-8', auto_save=False):
        """

        :param src_lang:
        :param encoding:
        :param auto_save:
        :return: Dictionary.
        """
        all_locales = listdir(self.locale_dir)
        locales = [locale for locale in all_locales if locale in SUPPORTED_LANGUAGES]
        unsupported_locales = [locale for locale in all_locales if locale not in SUPPORTED_LANGUAGES]
        print(_('Attempting to translate the supported locales:\n{0}').format(', '.join(locales)))
        if unsupported_locales:
            print(_('The following locales are not yet supported by potranslator and will not be translated:\n{0}').format(', '.join(locales)))
        results = defaultdict(dict)
        for locale in locales:
            po_files = [file for file in listdir(join(self.locale_dir, locale, 'LC_MESSAGES')) if file.endswith('.po')]
            for po_file in po_files:
                path = join(self.locale_dir, locale, 'LC_MESSAGES', po_file)
                results[locale][po_file] = self.translate(path, src_lang=src_lang, target_lang=locale, encoding=encoding, auto_save=auto_save)
        return results

    def translate_from_pot(self, filename, target_langs, src_lang='auto', encoding='utf-8', auto_save=False):
        """

        :param filename:
        :param target_langs:
        :param src_lang:
        :param encoding:
        :param auto_save:
        :return: Dictionary.
        """
        pot = polib.pofile(filename, encoding=encoding)
        results = {}
        for target_lang in target_langs:
            po_file_name = filename.split('/')[-1].split('\\')[-1][:-1]
            po_path = join(self.locale_dir, '/'.join((target_lang, 'LC_MESSAGES', po_file_name)))
            po_dir = join(self.locale_dir, '/'.join((target_lang, 'LC_MESSAGES')))
            if not isfile(po_path):
                if not exists(po_dir):
                    makedirs(po_dir)
                po = deepcopy(pot)
                po.save(po_path)
            results[target_lang] = self.translate(po_path, target_lang=target_lang, src_lang=src_lang, encoding=encoding, auto_save=auto_save)
        return results

    def translate_all_pot(self, target_langs, src_lang='auto', encoding='utf-8', auto_save=False):
        """

        :param target_langs:
        :param src_lang:
        :param encoding:
        :param auto_save:
        :return: Dictionary.
        """
        pot_files = [file for file in listdir(self.pot_dir) if file.endswith('.pot')]
        results = {}
        for pot_file in pot_files:
            path = join(self.pot_dir, pot_file)
            results[pot_file] = self.translate_from_pot(path, target_langs=target_langs, src_lang=src_lang, encoding=encoding, auto_save=auto_save)
        return results
